var _ = require('lodash');
var Jsonix = require('jsonix').Jsonix;
var pd = require('pretty-data').pd;

var fs = require('fs');

var XLink_1_0 = require('w3c-schemas').XLink_1_0;
var OWS_1_1_0 = require('ogc-schemas').OWS_1_1_0;
var GML_3_1_1 = require('ogc-schemas').GML_3_1_1;
var SMIL_2_0 = require('ogc-schemas').SMIL_2_0;
var SMIL_2_0_Language = require('ogc-schemas').SMIL_2_0_Language;
var WMTS_1_0 = require('ogc-schemas').WMTS_1_0;



function createUnmarshaller() {
	var context =  new Jsonix.Context([XLink_1_0, OWS_1_1_0, GML_3_1_1, SMIL_2_0, SMIL_2_0_Language, WMTS_1_0]);
	return context.createUnmarshaller();
}

function createMarshaller(options) {
	var context =  new Jsonix.Context([XLink_1_0, OWS_1_1_0, GML_3_1_1, SMIL_2_0, SMIL_2_0_Language, WMTS_1_0], options);
	return context.createMarshaller();
}

function findNamespaces(doc) {
	// Start with defaults
	var namespaces = {
		'http://www.w3.org/1999/xlink': 'xlink'
	};

	function findDefinedNamespaces(d) {
		// Only objects may contain a namespace
		if (_.isObject(d)) {
			if (_.isString(d.namespaceURI) && d.namespaceURI !== '') {
				namespaces[d.namespaceURI] = d.prefix;
			}
		}

		// Look for underlying objects
		if (_.isObject(d) || _.isArray(d)) {
			_.each(d, function(obj, key) {
				findDefinedNamespaces(obj);
			});
		}
	}

	findDefinedNamespaces(doc);

	return {
		namespacePrefixes: namespaces
	};
}

var unmarshaller = createUnmarshaller();

var sourceStr = fs.readFileSync('./source.xml').toString();
var doc = unmarshaller.unmarshalString(sourceStr);

var namespaces = findNamespaces(doc);
// NOTE! By uncommenting the next line, the marshaller will not be supplied with namespace prefix information and the Value element will get a proper namespace.
//namespaces = undefined;
var marshaller = createMarshaller(namespaces);

var outStr = marshaller.marshalString(doc);
outStr = pd.xml(outStr);

console.log(outStr);
if (outStr.indexOf('<Value') !== -1) {
	console.error('### Broken document!');
} else {
	console.error('### Document seems fine...');
}
